%  Scratch Analysis from 2 spots regions
%
%
%  Installation:
%
%  - Copy this file into the XTensions folder in the Imaris installation directory
%  - You will find this function in the Image Processing menu
%
%    <CustomTools>
%      <Menu>
%       <Submenu name="Spots Functions">
%        <Item name="Spots Scratch2" icon="Matlab">
%          <Command>MatlabXT::SpotsScratch2(%i)</Command>
%        </Item>
%       </Submenu>
%      </Menu>
%    </CustomTools>
%
%
%  Description:
%
%

function SpotsScratch2(aImarisApplicationID)

% connect to Imaris interface
if ~isa(aImarisApplicationID, 'Imaris.IApplicationPrxHelper')
    javaaddpath ImarisLib.jar
    vImarisLib = ImarisLib;
    if ischar(aImarisApplicationID)
        aImarisApplicationID = round(str2double(aImarisApplicationID));
    end
    vImarisApplication = vImarisLib.GetApplication(aImarisApplicationID);
else
    vImarisApplication = aImarisApplicationID;
end

% the user has to create a scene with some spots
vSurpassScene = vImarisApplication.GetSurpassScene;
if isequal(vSurpassScene, [])
    msgbox('Please create some Spots in the Surpass scene!');
    return;
end

% get the spots
vSpots = vImarisApplication.GetSurpassSelection;
vSpotsSelected = vImarisApplication.GetFactory.IsSpots(vSpots);

if vSpotsSelected
    vScene = vSpots.GetParent;
else
    vScene = vImarisApplication.GetSurpassScene;
end
vNumberOfSpots = 0;

% find datacontainer of children
for vChildIndex = 1:vScene.GetNumberOfChildren
    vDataItem = vScene.GetChild(vChildIndex - 1);
    % check if dataitem is a Data container
    if vImarisApplication.GetFactory.IsDataContainer(vDataItem)
        vDataContainer = vImarisApplication.GetFactory.ToDataContainer(vDataItem);
        % check all items in the data container
        for vNewChildIndex = 1:vDataContainer.GetNumberOfChildren
            vNewDataItem = vDataContainer.GetChild(vNewChildIndex - 1);
            if vImarisApplication.GetFactory.IsSpots(vNewDataItem)
                vNumberOfSpots = vNumberOfSpots+1;
                vSpotsList{vNumberOfSpots} = vImarisApplication.GetFactory.ToSpots(vNewDataItem); %#ok<AGROW>
                vNamesList{vNumberOfSpots} = char(vNewDataItem.GetName); %#ok<AGROW>
            end
        end
        % if it is not a data container, it may be a spots
    else
        if vImarisApplication.GetFactory.IsSpots(vDataItem)
            vNumberOfSpots = vNumberOfSpots+1;
            vSpotsList{vNumberOfSpots} = vImarisApplication.GetFactory.ToSpots(vDataItem);
            vNamesList{vNumberOfSpots} = char(vDataItem.GetName);
        end
    end
end

if vNumberOfSpots<2
    msgbox('Please create at least 2 spots objects!');
    return;
end

vNamesList = vNamesList(1:vNumberOfSpots);

vPair = [];
while length(vPair) ~= 2
    [vPair, vOk] = listdlg('ListString',vNamesList,'SelectionMode','multiple',...
        'ListSize',[250 150],'Name','Colocalize spots','InitialValue',[1,2], ...
        'PromptString',{'Please select the 2 spots groups to analyze:'});
    if vOk<1, return, end
    if length(vPair) ~= 2
        vHandle = msgbox(['Please select two (2) objects. Use "Control" and left ', ...
            'click to select/unselect an object of the list.']);
        uiwait(vHandle);
    end
end

vSpots1 = vSpotsList{vPair(1)};
vSpots2 = vSpotsList{vPair(2)};

T1 = vSpots1.GetIndicesT + 1;
T2 = vSpots2.GetIndicesT + 1;
spotValues1 = vSpots1.Get();
spotValues2 = vSpots2.Get();
xyz1 = spotValues1.mPositionsXYZ;
xyz2 = spotValues2.mPositionsXYZ;

objectIDs1 = vSpots1.GetTrackIds;
objectEdges1 = vSpots1.GetTrackEdges;
trackLabels1 = unique(objectIDs1);

objectIDs2 = vSpots2.GetTrackIds;
objectEdges2 = vSpots2.GetTrackEdges;
trackLabels2 = unique(objectIDs2);



if nargin <= 2
    % Prompt the distance for the current time point
    x1 = xyz1(T1==vImarisApplication.GetVisibleIndexT,:);
    x2 = xyz2(T2==vImarisApplication.GetVisibleIndexT,:);
    xyz = [x1;x2];
    [~,dist] = clusterXY(xyz,0,0);
end

hw = waitbar(0,'Finding Gap Spots for each Time Points');
vSpotsXYZ = [];
vSpotsTime = [];
for i = 1:max(T1)
    waitbar(double(i)/max(double(T1)),hw);
    x1 = xyz1(T1==i,1:2);
    x2 = xyz2(T2==i,1:2);
    mi = min([x1;x2]);
    mx = max([x1;x2] + dist);
    [X,Y]= meshgrid(mi(1):dist:mx(1),mi(2):dist:mx(2));
    NX = X(:);
    NY = Y(:);
    NXY = [NX NY];
    dd = distanceDiff(NXY,x1,x2);
    vSpotsXYZ = [vSpotsXYZ;NXY(dd<dist,:) repmat(mean(xyz1(:,3)),sum(dd<dist),1)]; %#ok<AGROW>
    vSpotsTime = [vSpotsTime;repmat(i-1,sum(dd<dist),1)]; %#ok<AGROW>
end
vRadii = repmat(dist/5,size(vSpotsTime,1),1);
vNewSpots = vImarisApplication.GetFactory.CreateSpots;
vNewSpots.Set(vSpotsXYZ, vSpotsTime, vRadii);
vNewSpots.SetName('Gap Spots');

hold on
% plotting group 1
xy = struct('x',[],'y',[]);
for i = 1:size(trackLabels1,1)
rEdges1 = objectEdges1(objectIDs1 == trackLabels1(i), :);
rSpots1 = unique(rEdges1);
st = T1(rSpots1+1);
% find the point of center of scratch which is the closet to the cell
p = vSpotsXYZ(vSpotsTime==st(1)-1,:);
rPos1 = xyz1(rSpots1 + 1, :);
np = repmat(rPos1(1,:),[size(p,1) 1]);
dd = sqrt(sum((np - p).^2,2));
[~,indmin] = min(dd);
% Select this vector as the direction
if ~isempty(indmin)
    du = p(indmin,1:2)-rPos1(1,1:2);
    du = du/sqrt(sum(du.^2));
    dv = [du(2);-du(1)];
    rPos1 = rPos1 - repmat(rPos1(1,:),[size(rPos1,1) 1]);
    plot(dv'*rPos1(:,1:2)',du*rPos1(:,1:2)','-k')
    xy(i).x = dv'*rPos1(:,1:2)';
    xy(i).y = du*rPos1(:,1:2)';
end
end
% plotting group 2
for i = 1:size(trackLabels2,1)
rEdges2 = objectEdges2(objectIDs2 == trackLabels2(i), :);
rSpots2 = unique(rEdges2);
st = T2(rSpots2+1);
% find the point of center of scratch which is the closet to the cell
p = vSpotsXYZ(vSpotsTime==st(1)-1,:);
rPos2 = xyz2(rSpots2 + 1, :);
np = repmat(rPos2(1,:),[size(p,1) 1]);
dd = sqrt(sum((np - p).^2,2));
[~,indmin] = min(dd);
% Select this vector as the direction
if ~isempty(indmin)
    du = p(indmin,1:2)-rPos2(1,1:2);
    du = du/sqrt(sum(du.^2));
    dv = [du(2);-du(1)];
    rPos2 = rPos2 - repmat(rPos2(1,:),[size(rPos2,1) 1]);
    plot(dv'*rPos2(:,1:2)',du*rPos2(:,1:2)','-k')
    xy(i+size(trackLabels1,1)).x = dv'*rPos1(:,1:2)';
    xy(i+size(trackLabels1,1)).y = du*rPos1(:,1:2)';
end
end

vFileName = vImarisApplication.GetCurrentFileName;
matFileName = strrep(char(vFileName),'.ims','_first.mat');
save(matFileName,'xy');

ax = axis();
axmax = max(abs(ax));
axis equal
axis([-axmax axmax -axmax axmax]);



figure,
hold on
% plotting group 1
xy = struct('x',[],'y',[]);
for i = 1:size(trackLabels1,1)
rEdges1 = objectEdges1(objectIDs1 == trackLabels1(i), :);
rSpots1 = unique(rEdges1);
st = T1(rSpots1+1);
% find the point of center of scratch which is the closet to the cell
p = vSpotsXYZ(vSpotsTime==st(end)-1,:);
rPos1 = xyz1(rSpots1 + 1, :);
np = repmat(rPos1(end,:),[size(p,1) 1]);
dd = sqrt(sum((np - p).^2,2));
[~,indmin] = min(dd);
% Select this vector as the direction
if ~isempty(indmin)
    du = p(indmin,1:2)-rPos1(end,1:2);
    du = du/sqrt(sum(du.^2));
    dv = [du(2);-du(1)];
    rPos1 = rPos1 - repmat(rPos1(1,:),[size(rPos1,1) 1]);
    plot(dv'*rPos1(:,1:2)',du*rPos1(:,1:2)','-k')
    xy(i).x = dv'*rPos1(:,1:2)';
    xy(i).y = du*rPos1(:,1:2)';
end
end


% plotting group 2
for i = 1:size(trackLabels2,1)
rEdges2 = objectEdges2(objectIDs2 == trackLabels2(i), :);
rSpots2 = unique(rEdges2);
st = T2(rSpots2+1);
% find the point of center of scratch which is the closet to the cell
p = vSpotsXYZ(vSpotsTime==st(end)-1,:);
rPos2 = xyz2(rSpots2 + 1, :);
np = repmat(rPos2(end,:),[size(p,1) 1]);
dd = sqrt(sum((np - p).^2,2));
[~,indmin] = min(dd);
% Select this vector as the direction
if ~isempty(indmin)
    du = p(indmin,1:2)-rPos2(end,1:2);
    du = du/sqrt(sum(du.^2));
    dv = [du(2);-du(1)];
    rPos2 = rPos2 - repmat(rPos2(1,:),[size(rPos2,1) 1]);
    plot(dv'*rPos2(:,1:2)',du*rPos2(:,1:2)','-k')
    xy(i+size(trackLabels1,1)).x = dv'*rPos1(:,1:2)';
    xy(i+size(trackLabels1,1)).y = du*rPos1(:,1:2)';
end
end

vFileName = vImarisApplication.GetCurrentFileName;
matFileName = strrep(char(vFileName),'.ims','_last.mat');
save(matFileName,'xy');

ax = axis();
axmax = max(abs(ax));
axis equal
axis([-axmax axmax -axmax axmax]);


vSurpassScene.AddChild(vNewSpots, -1);
close(hw);

function dd = distanceDiff(NXY,xyz1,xyz2)

dd = zeros(size(NXY,1),1);
for i = 1:size(NXY,1)
    d = sqrt(sum((xyz2 - repmat(NXY(i,:),[size(xyz2,1) 1])).^2,2));
    d1 = min(d);
    d = sqrt(sum((xyz1 - repmat(NXY(i,:),[size(xyz1,1) 1])).^2,2));
    d2 = min(d);
    dd(i) = abs(d1-d2);
end

%% Function to cluster groups within a distance "dist" to each other

function [groupIND,dist] = clusterXY(xy,dist,timesDist)

D = newpdist(xy);
if ~isempty(D)
    % Calculate the average minimal distance between spots of the
    % first frame
    if dist <= 0
        di = logical(diag(ones(size(D,1),1)));
        D(di) = nan;
        distmin = mean(min(D));
        D(di) = 0;
        if dist == 0
            dist = inputdlg({'Enter max distance (actual value is average distance):'},...
                'Input',1,{num2str(distmin)});
        else
            dist = {num2str(timesDist*distmin)};
        end
        if isempty(dist)
            groupIND = zeros(size(D,1),1);
            return
        end
        dist = str2double(dist{1});
    end
    groupIND = zeros(size(D,1),1);
    list_ingroup = false(size(xy,1),1);
    list_done = false(size(xy,1),1);
    groupnum = 1;
    while ~all(list_done)
        todo = find(~list_done);
        todo = todo(1);
        currentgroupnum = groupnum;
        cindex = [];
        while groupnum == currentgroupnum
            index = find(D(todo,:) < dist);
            cindex = checkIndex(cindex,index);
            list_ingroup(index) = 1;
            list_done(todo) = 1;
            islast = find(cindex == todo);
            if islast == length(cindex)
                groupnum = groupnum + 1;
            else
                todo = cindex(islast + 1);
            end
        end
        for ii = 1:size(cindex,2)
            groupIND(cindex(ii)) = currentgroupnum;
        end
    end
else
    groupIND = [];
end

%--------------------------------------------------------------------------
% check if index is part of the list and update list
function newind = checkIndex(oldind,ind)

isin = false(length(ind),1);
for ii = 1:length(ind)
    isin(ii) = any(oldind == ind(ii));
end
newind = [oldind ind(~isin)];
%--------------------------------------------------------------------------
function D = newpdist(xy)

D = zeros(size(xy,1));
for ii = 1:size(xy,1)
    p1 = repmat(xy(ii,:),[size(xy,1) 1]);
    D(ii,:) = sqrt(sum((xy - p1).^2,2));
end

